<?php
/**
 *  XDbInfoView class file.
 *
 * @author Stefan Volkmar <volkmar_yii@email.de>
 * @version 1.6
 * @license BSD
 */

/**
 * This widget display all informations of the connected database in a
 * tabular manner.
 *
 * @author Stefan Volkmar <volkmar_yii@email.de>
 */

Yii::setPathOfAlias('XDbInfoView',dirname(__FILE__));

class XDbInfoView extends CWidget
{
	/**
	 * @var mixed the CSS file used for the widget. 
	 * If false, the default CSS file will be used. Otherwise, the specified CSS file
	 * will be included when using this widget.
	 */
    public $cssFile=false;

    public $usePlainView=false;

    protected $baseUrl;
	/**
	 * Initializes the widget.
	 * This method registers all needed client scripts 
	 */
	public function init()
	{
        parent::init();
        
        $this->baseUrl = CHtml::asset(dirname(__FILE__).'/assets');
        $cs=Yii::app()->getClientScript();
        $cs->registerCoreScript('jquery');
        if($this->cssFile===false)
            $this->cssFile = $this->baseUrl.'/css/xdbinfoview.css';
        $cs->registerCssFile($this->cssFile);

        $js = <<<EOT
$(".xdbinfo-stable-caption").click(function(){
    $(this).next(".xdbinfo-togglebox").slideToggle("slow");
});
EOT;

        $id=$this->getId();
        if (!$this->usePlainView)
            $cs->registerScript(__CLASS__.'#'.$id,$js);
        
	}

    public function run()
    {
        if ($this->usePlainView){
            self::renderPlainView();
        } else {
            self::renderToggleView();
        }
    }

    static protected function renderDbConnectionTable(&$db, &$htm){
        $htm .= '<table class="xdbinfo-ctable">';
        $htm .= '<caption>'.Yii::t("XDbInfoView.main","Informations about the database connection").'</caption>';
        $htm .= '<tbody>';
        $htm .= '<tr><td class="xdbinfo-ctable-firstCol">'.Yii::t("XDbInfoView.main","the charset used for connection").':</td><td>'.$db->charset.'</td></tr>';
        $htm .= '<tr><td class="xdbinfo-ctable-firstCol">'.Yii::t("XDbInfoView.main","the version information of the DB driver").':</td><td>'.$db->clientVersion.'</td></tr>';
        $htm .= '<tr><td class="xdbinfo-ctable-firstCol">'.Yii::t("XDbInfoView.main","name of the DB driver").':</td><td>'.$db->driverName.'</td></tr>';
        $htm .= '<tr><td class="xdbinfo-ctable-firstCol">'.Yii::t("XDbInfoView.main","the case of the column names").':</td><td>'.$db->columnCase.'</td></tr>';
        $htm .= '<tr><td class="xdbinfo-ctable-firstCol">'.Yii::t("XDbInfoView.main","how the null and empty strings are converted").':</td><td>'.$db->nullConversion.'</td></tr>';
        $htm .= '<tr><td class="xdbinfo-ctable-firstCol">'.Yii::t("XDbInfoView.main","the Data Source Name (DSN)").':</td><td>'.$db->connectionString.'</td></tr>';
        $htm .= '</tbody>';
        $htm .= '</table>';
        $htm .= '<br />';
    }

    static protected function renderPlainView() {
        $htm = '<h2>'.Yii::t("XDbInfoView.main","Informations about the database in your project with the connection ID").': db</h2>';
        try {
            $db=Yii::app()->db;
            self::renderDbConnectionTable($db, $htm);
            $htm .= '<table>';
            $htm .= '<caption>'.Yii::t("XDbInfoView.main","Informations about the schema").'</caption>';
            self::renderSchemaTableHeads($htm, TRUE);
            $htm .= '<tbody>';
            foreach($db->schema->tables as $table){
                $i=0;
                foreach($table->columns as $column){
                    $htm .= '<tr>';
                    $htm .= (!$i) ? '<td class="xdbinfo-stable-firstCol">'.$table->name.'</td>': '<td>&nbsp;</td>';
                    $htm .= '<td>'.$column->name.'</td>';
                    $htm .= '<td>'.$column->dbType.'</td>';
                    $htm .= '<td>'.(($column->isPrimaryKey)?Yii::t("XDbInfoView.main","yes"):Yii::t("XDbInfoView.main","no")).'</td>';
                    $htm .= '<td>'.(($column->isForeignKey)?Yii::t("XDbInfoView.main","yes"):Yii::t("XDbInfoView.main","no")).'</td>';
                    $htm .= '</tr>';
                    $i++;
                }
            }
            $htm .= '</tbody>';
            $htm .= '</table>';
        } catch (Exception $e) {
            self::renderNoDbMessage($htm, $e->getMessage());
        }
        echo $htm;
    }

    static protected function renderToggleView() {
        $htm = '<h2>'.Yii::t("XDbInfoView.main","Informations about the database in your project with the connection ID").': db</h2>';
        try {
            $db=Yii::app()->db;
            self::renderDbConnectionTable($db, $htm);
            $htm .= '<h3>'.Yii::t("XDbInfoView.main","Informations about the schema").'</h3>';
            foreach($db->schema->tables as $table){
                self::renderSchemaTableCaption($table, $htm);
                self::renderSchemaTable($table, $htm);
            }
        } catch (Exception $e) {
            //var_dump($e) ;
            self::renderNoDbMessage($htm, $e->getMessage());
        }
        echo $htm;
    }

    static protected function renderSchemaTable(&$table, &$htm) {

        $htm .= '<div class="xdbinfo-togglebox" style="display: none; ">';
        $htm .= '<div class="xdbinfo-block">';
        $htm .= '<table class="xdbinfo-stable">';
        self::renderSchemaTableHeads($htm);
        $htm .= '<tbody>';
        foreach($table->columns as $column){
            $htm .= '<tr>';
            $htm .= '<td>'.$column->name.'</td>';
            $htm .= '<td>'.$column->dbType.'</td>';
            $htm .= '<td>'.(($column->isPrimaryKey)?Yii::t("XDbInfoView.main","yes"):Yii::t("XDbInfoView.main","no")).'</td>';
            $htm .= '<td>'.(($column->isForeignKey)?Yii::t("XDbInfoView.main","yes"):Yii::t("XDbInfoView.main","no")).'</td>';
            $htm .= '</tr>';
        }

        $htm .= '</tbody>';
        $htm .= '</table>';
        $htm .= '</div></div>';

    }

    static protected function renderSchemaTableCaption(&$table, &$htm){
        $count=Yii::app()->db->createCommand()->select('count(*) as num')->from($table->name)->queryScalar();
        $rows=Yii::t("XDbInfoView.main","rows");
        $strCount="&nbsp;[&nbsp;$count&nbsp;$rows&nbsp;]";
        $htm .= '<div class="xdbinfo-stable-caption" title="'.Yii::t("XDbInfoView.main","Click me to show more or less!").'">'.Yii::t("XDbInfoView.main","table and/or view").':&nbsp;'.$table->name.$strCount.'</div>';
    }

    static protected function renderSchemaTableHeads(&$htm, $showTableName=false){
        $htm .= '<thead>';
        $htm .= '<tr>';
        if ($showTableName)
            $htm .= '<th>'.Yii::t("XDbInfoView.main","table and/or view").'</th>';
        $htm .= '<th>'.Yii::t("XDbInfoView.main","field name").'</th>';
        $htm .= '<th>'.Yii::t("XDbInfoView.main","type").'</th>';
        $htm .= '<th>'.Yii::t("XDbInfoView.main","is primary key").'</th>';
        $htm .= '<th>'.Yii::t("XDbInfoView.main","is foreign key").'</th>';
        $htm .= '</tr>';
        $htm .= '</thead>';

    }

    static protected function renderNoDbMessage(&$htm, $message){
        $htm .= '<p class="xdbinfo-warning">';        
        $htm .= Yii::t("XDbInfoView.main","You don't use a database connection in your project.");
        $htm .= '<br /><br />'.CHtml::encode($message);
        $htm .= '</p>';
    }
}
